import java.io.*;
import java.util.Vector;
import org.xmlpull.v1.*;
import org.kxml2.io.KXmlParser;

public class ContactItemParser {
    static final String CONTACTXML_NS = "http://www.xmlns.org/2002/ContactXML";
    ContactItem item;
    XmlPullParser xpp;

    public void setItem(ContactItem theItem) {
	item = theItem;
    }

    public void parse(InputStream s) throws Exception {
	xpp = null;
	xpp = new KXmlParser();
	xpp.setFeature(xpp.FEATURE_PROCESS_NAMESPACES, true);
	xpp.setInput(s, "SJIS");
	int pos[] = new int[2];
	int et = xpp.getEventType();
	if(et == xpp.START_DOCUMENT) {
	    et = xpp.next();
	} else {
	    throw new RuntimeException("not xml document");
	}
	while (et != xpp.END_DOCUMENT) {
	    if (et == xpp.START_TAG
		&& CONTACTXML_NS.equals(xpp.getNamespace())) {
		if (xpp.getName().equals("ContactXMLItem")) {
		    parseItem();
		    break; // : ǽΥƥषɤޤʤ
		}
	    }
	    et = xpp.next();
	}
    }
	
    protected void parseItem() throws Exception {
	int et = xpp.getEventType();
	while(et != xpp.END_DOCUMENT 
	      && ! (et == xpp.END_TAG 
		    && CONTACTXML_NS.equals(xpp.getNamespace())
		    && xpp.getName().equals("ContactXMLItem"))) {
	    if (CONTACTXML_NS.equals(xpp.getNamespace())) {
		if (et == xpp.START_TAG) {
		    if(xpp.getName().equals("FullName")) {
			item.pronunciation =
			    xpp.getAttributeValue(null, "pronunciation"); 
			if(xpp.next() == xpp.TEXT) {
			    item.name = xpp.getText();
			}
		    } else if (xpp.getName().equals("AddressItem")) {
			parseAddress();
		    } else if (xpp.getName().equals("OccupationItem")) {
			parseOccupation();
		    } else if (xpp.getName().equals("PhoneItem")) {
			if(xpp.next() == xpp.TEXT) {
			    item.phones.addElement(xpp.getText());
			}
		    } else if (xpp.getName().equals("EmailItem")) {
			if(xpp.next() == xpp.TEXT) {
			    item.emails.addElement(xpp.getText());
			}
		    } else if (xpp.getName().equals("WebItem")) {
			if(xpp.next() == xpp.TEXT) {
			    item.webs.addElement(xpp.getText());
			}
		    } else if (xpp.getName().equals("ImageItem")) {
			if (item.imageURL == null && "Portrait".equals(
			      xpp.getAttributeValue(null, "imageSemantics"))) {
			    // : ǽδ̿
			    item.imageURL = 
				xpp.getAttributeValue(null, "url"); 
			}
		    }
		}
	    }
	    et = xpp.next();
	}
    }

    void parseAddress() throws Exception {
	String address;

	String locType = xpp.getAttributeValue(null, "locationType");
	if ("Home".equals(locType)) {
	    address = "[] ";
	} else if ("Office".equals(locType)) {
	    address = "[] ";
	} else {
	    address = "";
	}

	String country = "";
	String pref = "";
	String city = "";
	String town = "";
	String street = "";
	String building = "";
	String pob = "";
	String other = "";
	String unknown = "";

	int et = xpp.next();
	while(et != xpp.END_DOCUMENT 
	      && ! (et == xpp.END_TAG 
		    && CONTACTXML_NS.equals(xpp.getNamespace())
		    && xpp.getName().equals("AddressItem"))) {
	    if (CONTACTXML_NS.equals(xpp.getNamespace())) {
		if (et == xpp.START_TAG
		    && xpp.getName().equals("AddressLine")) {
		    String type = 
			xpp.getAttributeValue(null, "addressLineType"); 
		    String value = "";
		    if(xpp.next() == xpp.TEXT) {
			value = xpp.getText() + "\n";
		    }
		    if ("Country".equals(type)) {
			country = value;
		    } else if ("Prefecture".equals(type)) {
			pref = value;
		    } else if ("City".equals(type)) {
			city = value;
		    } else if ("Town".equals(type)) {
			town = value;
		    } else if ("Number".equals(type)) {
			street = value;
		    } else if ("Building".equals(type)) {
			building = value;
		    } else if ("POB".equals(type)) {
			pob = value;
		    } else if ("Others".equals(type)) {
			other = value;
		    } else if ("Unknown".equals(type)) {
			unknown = value;
		    }
		}
	    }
	    et = xpp.next();
	}
	address = country + pref + city + town + street + building 
	    + pob + other + unknown;
	item.address.addElement(address);
    }
    void parseOccupation() throws Exception {
	String occupation = "";

	int et = xpp.next();
	while(et != xpp.END_DOCUMENT 
	      && ! (et == xpp.END_TAG 
		    && CONTACTXML_NS.equals(xpp.getNamespace())
		    && xpp.getName().equals("OccupationItem"))) {
	    if (CONTACTXML_NS.equals(xpp.getNamespace())) {
		if (et == xpp.START_TAG) {
		    String tagname = xpp.getName();
		    if(xpp.next() == xpp.TEXT) {
			// children of OccupationItem comes in order.
			// so, i need not distinguish tagnames.
			occupation += xpp.getText() + "\n";
		    }
		}
	    }
	    et = xpp.next();
	}
	item.occupation.addElement(occupation);
    }
}
